/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        Copyright (c) Non, Inc. 1998 -- All Rights Reserved

PROJECT:        C Compiler
MODULE:         GnuCEmitter_print
FILE:           GnuCEmitter_print.g

AUTHOR:         Monty Zukowski (jamz@cdsnet.net) April 28, 1998

DESCRIPTION:

                This tree grammar is for a Gnu C AST.
                It turns the tree back into source code.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

{
import java.io.*;
import java.util.*;

import antlr.CommonAST;
import antlr.DumpASTVisitor;
}

class GnuCEmitter_print extends GnuCTreeParser;

options
        {
        importVocab = GNUC;
        buildAST = false;
        ASTLabelType = "TNode";

        // Copied following options from java grammar.
        codeGenMakeSwitchThreshold = 2;
        codeGenBitsetTestThreshold = 3;
        }

{

int tabs = 0;
PrintStream currentOutput = System.out;
int lineNum = 1;
String currentSource = "";
LineObject trueSourceFile;
final int lineDirectiveThreshold = Integer.MAX_VALUE;
PreprocessorInfoChannel preprocessorInfoChannel = null;
Stack sourceFiles = new Stack();

GnuCEmitter_print( PreprocessorInfoChannel preprocChannel )
{
        preprocessorInfoChannel = preprocChannel;
}

void initializePrinting()
{
    Vector preprocs = preprocessorInfoChannel.extractLinesPrecedingTokenNumber( new Integer(1) );
    printPreprocs(preprocs);
/*    if ( currentSource.equals("") ) {
        trueSourceFile = new LineObject(currentSource);
        currentOutput.println("# 1 \"" + currentSource + "\"\n");
        sourceFiles.push(trueSourceFile);
    }
*/
}

void finalizePrinting() {
    // flush any leftover preprocessing instructions to the stream

    printPreprocs(
        preprocessorInfoChannel.extractLinesPrecedingTokenNumber(
                new Integer( preprocessorInfoChannel.getMaxTokenNumber() + 1 ) ));
    //print a newline so file ends at a new line
    currentOutput.println();
}

void printPreprocs( Vector preprocs )
{
    // if there was a preprocessingDirective previous to this token then
    // print a newline and the directive, line numbers handled later
    if ( preprocs.size() > 0 ) {
        if ( trueSourceFile != null ) {
            currentOutput.println();  //make sure we're starting a new line unless this is the first line directive
        }
        lineNum++;
        Enumeration e = preprocs.elements();
        while (e.hasMoreElements())
        {
            Object o = e.nextElement();
            if ( o.getClass().getName().equals("LineObject") ) {
                LineObject l = (LineObject) o;

                // we always return to the trueSourceFile, we never enter it from another file
                // force it to be returning if in fact we aren't currently in trueSourceFile
                if (( trueSourceFile != null ) //trueSource exists
                        && ( !currentSource.equals(trueSourceFile.getSource()) ) //currently not in trueSource
                        && ( trueSourceFile.getSource().equals(l.getSource())  ) ) { //returning to trueSource
                    l.setEnteringFile( false );
                    l.setReturningToFile( true );
                }

                // print the line directive
                currentOutput.println(l);
                lineNum = l.getLine();
                currentSource = l.getSource();

                // the very first line directive always represents the true sourcefile
                if ( trueSourceFile == null ) {
                    trueSourceFile = new LineObject(currentSource);
                    sourceFiles.push(trueSourceFile);
                }

                // keep our own stack of files entered
                if ( l.getEnteringFile() ) {
                    sourceFiles.push(l);
                }

                // if returning to a file, pop the exited files off the stack
                if ( l.getReturningToFile() ) {
                    LineObject top = (LineObject) sourceFiles.peek();
                    while (( top != trueSourceFile ) && (! l.getSource().equals(top.getSource()) )) {
                        sourceFiles.pop();
                        top = (LineObject) sourceFiles.peek();
                    }
                }
            }
            else { // it was a #pragma or such
                currentOutput.println(o);
                lineNum++;
            }
        }
    }

}

void print( TNode t ) {
    int tLineNum = t.getLocalLineNum();
    if ( tLineNum == 0 ) tLineNum = lineNum;

    Vector preprocs = preprocessorInfoChannel.extractLinesPrecedingTokenNumber((Integer)t.getAttribute("tokenNumber"));
    printPreprocs(preprocs);

    if ( (lineNum != tLineNum) ) {
        // we know we'll be newlines or a line directive or it probably
        // is just the case that this token is on the next line
        // either way start a new line and indent it
        currentOutput.println("\"nl1\", ");
        lineNum++;
        printTabs();
    }

    if ( lineNum == tLineNum ){
        // do nothing special, we're at the right place
    }
    else {
        int diff = tLineNum - lineNum;
        if ( lineNum < tLineNum ) {
            // print out the blank lines to bring us up to right line number
            for ( ; lineNum < tLineNum ; lineNum++ ) {
                currentOutput.println("\"nl2\", ");
            }
            printTabs();
        }
        else { // just reset lineNum
            lineNum = tLineNum;
        }
    }
    currentOutput.print( "\"" + t.getText() + "\", " );
//    currentOutput.print( t.getText() + " " );
}

/* This was my attempt at being smart about line numbers
   It didn't work quite right but I don't know why, I didn't
   have enough test cases.  Worked ok compiling rcs and ghostscript
*/
void printAddingLineDirectives( TNode t ) {
    int tLineNum = t.getLocalLineNum();
    String tSource = (String) t.getAttribute("source");

    if ( tSource == null ) tSource = currentSource;
    if ( tLineNum == 0 ) tLineNum = lineNum;

    Vector preprocs = preprocessorInfoChannel.extractLinesPrecedingTokenNumber((Integer)t.getAttribute("tokenNumber"));
    printPreprocs(preprocs);

    if ( (lineNum != tLineNum) || !currentSource.equals(tSource) ) {
        // we know we'll be newlines or a line directive or it probably
        // is just the case that this token is on the next line
        // either way start a new line and indent it
        currentOutput.println("\"nl3\", ");
        lineNum++;
        printTabs();
    }

    if ( ( lineNum == tLineNum ) && ( currentSource.equals(tSource) ) ){
        // do nothing special, we're at the right place
    }
    else if ( currentSource.equals(tSource) ) {
        int diff = tLineNum - lineNum;
        if (diff > 0 && diff < lineDirectiveThreshold) {
            // print out the blank lines to bring us up to right line number
            for ( ; lineNum < tLineNum ; lineNum++ ) {
                currentOutput.println("\"nl4\", ");
            }
        }
        else { // print line directive to get us to right line number
            // preserve flags 3 and 4 if present in current file
            if ( ! sourceFiles.empty() ) {
                LineObject l = (LineObject) sourceFiles.peek();
                StringBuffer tFlags = new StringBuffer("");
                if (l.getSystemHeader()) {
                    tFlags.append(" 3");
                }
                if (l.getTreatAsC()) {
                    tFlags.append(" 4");
                }
                currentOutput.println("# " + tLineNum + " \"" + tSource + "\"" + tFlags.toString());
                lineNum = tLineNum;
            }
        }

        printTabs();
    }
    else { // different source
        Enumeration sources = sourceFiles.elements();
        // see if we're returning to a file we entered earlier
        boolean returningToEarlierFile = false;
        while (sources.hasMoreElements()) {
            LineObject l = (LineObject) sources.nextElement();
            if (l.getSource().equals(tSource)) {
                returningToEarlierFile = true;
                break;
            }
        }
        if (returningToEarlierFile) {
            // pop off the files we're exiting, but never pop the trueSourceFile
            LineObject l = (LineObject) sourceFiles.peek();
            while ( ( l != trueSourceFile ) &&(! l.getSource().equals(tSource) ) ) {
                sourceFiles.pop();
                l = (LineObject) sourceFiles.peek();
            }

            // put in the return flag, plus others as needed
            StringBuffer tFlags = new StringBuffer(" 2");
            if (l.getSystemHeader()) {
                tFlags.append(" 3");
            }
            if (l.getTreatAsC()) {
                tFlags.append(" 4");
            }

            currentOutput.println("# " + tLineNum + " \"" + tSource + "\"" + tFlags);
            lineNum = tLineNum;
            currentSource = tSource;
            printTabs();
        }
        else {  // entering a file that wasn't in the original source
                // pretend we're entering it from top of stack
            currentOutput.println("# " + tLineNum + " \"" + tSource + "\"" + " 1");
            lineNum = tLineNum;
            currentSource = tSource;
            printTabs();
        }
    }
    currentOutput.print( "\"" + t.getText() + "\" " );
}

/** It is not ok to print newlines from the String passed in as
it will screw up the line number handling **/
void print( String s ) {
    currentOutput.print( "\"" + s + "\", ");
//    currentOutput.print( s + " " );
}

void printTabs() {
    for ( int i = 0; i< tabs; i++ ) {
        currentOutput.print( "\"\t\"," );
//        currentOutput.print( "\t" );
    }
}

void commaSep( TNode t ) {
    print( t );
    if ( t.getNextSibling() != null ) {
        print( "," );
    }
}

        int traceDepth = 0;
        public void reportError(RecognitionException ex) {
          if ( ex != null)   {
                System.err.println("ANTLR Tree Parsing RecognitionException Error: " + ex.getClass().getName() + " " + ex );
                ex.printStackTrace(System.err);
          }
        }
        public void reportError(NoViableAltException ex) {
                System.err.println("ANTLR Tree Parsing NoViableAltException Error: " + ex.toString());
                TNode.printTree( ex.node );
                ex.printStackTrace(System.err);
        }
        public void reportError(MismatchedTokenException ex) {
          if ( ex != null)   {
                TNode.printTree( ex.node );
                System.err.println("ANTLR Tree Parsing MismatchedTokenException Error: " + ex );
                ex.printStackTrace(System.err);
          }
        }
        public void reportError(String s) {
                System.err.println("ANTLR Error from String: " + s);
        }
        public void reportWarning(String s) {
                System.err.println("ANTLR Warning from String: " + s);
        }
        protected void match(AST t, int ttype) throws MismatchedTokenException {
                //System.out.println("match("+ttype+"); cursor is "+t);
                super.match(t, ttype);
        }
        public void match(AST t, BitSet b) throws MismatchedTokenException {
                //System.out.println("match("+b+"); cursor is "+t);
                super.match(t, b);
        }
        protected void matchNot(AST t, int ttype) throws MismatchedTokenException {
                //System.out.println("matchNot("+ttype+"); cursor is "+t);
                super.matchNot(t, ttype);
                }
        public void traceIn(String rname, AST t) {
          traceDepth += 1;
          for (int x=0; x<traceDepth; x++) System.out.print(" ");
          super.traceIn(rname, t);
        }
        public void traceOut(String rname, AST t) {
          for (int x=0; x<traceDepth; x++) System.out.print(" ");
          super.traceOut(rname, t);
          traceDepth -= 1;
        }

}

translationUnit  options {
  defaultErrorHandler=false;
}
        :
                                { initializePrinting(); }
               ( externalList )?
                                { finalizePrinting(); }
        ;



externalList
  { System.out.print( "(" ); print( "externalList" ); }
        :  (
                ( externalDef )+
           )
  { System.out.print( "), " ); }
        ;
externalDef
  { System.out.print( "(" ); print( "externalDef" ); }
        :  (
                declaration
        |       functionDef
        |       asm_expr
        |       typelessDeclaration
        |       s:SEMI                          { print( "SEMI" ); }
           )
  { System.out.print( "), " ); }
        ;
typelessDeclaration
  { System.out.print( "(" ); print( "typelessDeclaration WARNING" ); }
        :  (
                #(NTypeMissing initDeclList s: SEMI)    { print( "SEMI" ); }
           )
  { System.out.print( "), " ); }
        ;
asm_expr
  { System.out.print( "(" ); print( "asm_expr WARNING" ); }
        :  (
                #( a:"asm"                              { print( a ); }
                 ( v:"volatile"                         { print( v ); }
                 )?
                    lc:LCURLY                           { print( lc ); tabs++; }
                    expr
                    rc:RCURLY                           { tabs--; print( rc ); }
                    s:SEMI                              { print( "SEMI" ); }
                )
           )
  { System.out.print( "), " ); }
        ;
declaration
  { System.out.print( "(" ); print( "declaration" ); }
        :  (
                #( NDeclaration
                    declSpecifiers
                    (
                        initDeclList
                    )?
                    ( s:SEMI { print( "SEMI" ); } )+
                )
           )
  { System.out.print( "), " ); }
        ;
declSpecifiers
  { System.out.print( "(" ); print( "declSpecifiers" ); }
        :  (
                ( storageClassSpecifier
                | typeQualifier
                | typeSpecifier
                )+
           )
  { System.out.print( "), " ); }
        ;
storageClassSpecifier
  { System.out.print( "(" ); print( "storageClassSpecifier" ); }
        :  (
                a:"auto"                                { print( a ); }
        |       b:"register"                    { print( b ); }
        |       c:"typedef"                     { print( c ); }
        |       functionStorageClassSpecifier
           )
  { System.out.print( "), " ); }
        ;
functionStorageClassSpecifier
  { System.out.print( "(" ); print( "functionStorageClassSpecifier" ); }
        :  (
                a:"extern"
        |       b:"static"
        |       c:"inline"
           )
  { System.out.print( "), " ); }
        ;
typeQualifier
  { System.out.print( "(" ); print( "typeQualifier" ); }
        :  (
                a:"const"
        |       b:"volatile"
           )
  { System.out.print( "), " ); }
        ;
typeSpecifier
  { System.out.print( "(" ); print( "typeSpecifier" ); }
        :  (
                a:"void"                        { print( a ); }
        |       b:"char"                        { print( b ); }
        |       c:"short"                       { print( c ); }
        |       d:"int"                         { print( d ); }
        |       e:"long"                        { print( e ); }
        |       f:"float"                       { print( f ); }
        |       g:"double"                      { print( g ); }
        |       h:"signed"                      { print( h ); }
        |       i:"unsigned"                    { print( i ); }
        |       structSpecifier ( attributeDecl )*
        |       unionSpecifier  ( attributeDecl )*
        |       enumSpecifier
        |       typedefName
        |       #(n:"typeof" lp:LPAREN             { print( n ); print( "LPARENS" ); }
                    ( (typeName )=> typeName
                    | expr
                    )
                    rp:RPAREN                      { print( "RPARENS" ); }
                )
        |       p:"__complex"                   { print( p ); }
           )
  { System.out.print( "), " ); }
        ;
typedefName
  { System.out.print( "(" ); print( "typedefName" ); }
        :  (
                #(NTypedefName i:ID         { print( i ); } )
           )
  { System.out.print( "), " ); }
        ;
structSpecifier
  { System.out.print( "(" ); print( "structSpecifier" ); }
        :  (
            #( a:"struct"                       { print( a ); }
                structOrUnionBody
            )
           )
  { System.out.print( "), " ); }
        ;
unionSpecifier
  { System.out.print( "(" ); print( "unionSpecifier" ); }
        :  (
            #( a:"union"                        { print( a ); }
                structOrUnionBody
            )
           )
  { System.out.print( "), " ); }
        ;
structOrUnionBody
  { System.out.print( "(" ); print( "structOrUnionBody" ); }
        :  (
                ( (ID LCURLY) => i1:ID lc1:LCURLY   {  currentOutput.print("(\"structOrUnionBody.0\", "); print( i1 ); currentOutput.print("), ");  print( "{" ); tabs++; }
                        ( structDeclarationList )?
                        rc1:RCURLY                  { tabs--; print( rc1 ); }
                |   lc2:LCURLY                      {  currentOutput.print("(\"structOrUnionBody.2\", "); print( "{" ); currentOutput.print("), ");  tabs++; }
                    ( structDeclarationList )?
                    rc2:RCURLY                      { tabs--; print( rc2 ); }
                | i2:ID                     {  currentOutput.print("(\"structOrUnionBody.4\", "); print( i2 ); currentOutput.print("), ");  }
                )
           )
  { System.out.print( "), " ); }
        ;
structDeclarationList
  { System.out.print( "(" ); print( "structDeclarationList" ); }
        :  (
                ( structDeclaration             { print( "SEMI" ); }
                )+
           )
  { System.out.print( "), " ); }
        ;
structDeclaration
  { System.out.print( "(" ); print( "structDeclaration" ); }
        :  (
                specifierQualifierList structDeclaratorList
           )
  { System.out.print( "), " ); }
        ;
specifierQualifierList
  { System.out.print( "(" ); print( "specifierQualifierList" ); }
        :  (
                (
                typeSpecifier
                | typeQualifier
                )+
           )
  { System.out.print( "), " ); }
        ;
structDeclaratorList
  { System.out.print( "(" ); print( "structDeclaratorList" ); }
        :  (
                structDeclarator
                ( { print( "COMMA" ); } structDeclarator )*
           )
  { System.out.print( "), " ); }
        ;
structDeclarator
  { System.out.print( "(" ); print( "structDeclarator" ); }
        :  (
         
        #( NStructDeclarator
            ( declarator )?
            ( c:COLON { print( c ); } expr )?
            ( attributeDecl )*
        )
           )
  { System.out.print( "), " ); }
        ;
enumSpecifier
  { System.out.print( "(" ); print( "enumSpecifier" ); }
        :  (
            #(  a:"enum"                        { print( a ); }
                ( i:ID {  currentOutput.print("(\"enumSpecifier.1\", "); print( i ); currentOutput.print("), ");  } )?
                ( lc:LCURLY                        { print( lc ); tabs++; }
                    enumList
                  rc:RCURLY                        { tabs--; print( rc ); }
                )?
            )
           )
  { System.out.print( "), " ); }
        ;
enumList
  { System.out.print( "(" ); print( "enumList" ); }
        :  (
         
                enumerator ( { print( "COMMA" ); } enumerator)*
           )
  { System.out.print( "), " ); }
        ;
enumerator
  { System.out.print( "(" ); print( "enumerator" ); }
        :  (
                i:ID            { print( i ); }
                ( b:ASSIGN      { print( b ); }
                  expr
                )?
           )
  { System.out.print( "), " ); }
        ;
attributeDecl
  { System.out.print( "(" ); print( "attributeDecl WARNING" ); }
        :  (
         
        #( a:"__attribute"            { print( a ); }
           (b:. { print( b ); } )*
        )
        | #( n:NAsmAttribute            { print( n ); }
             lp:LPAREN                  { print( "LPARENS" ); }
             expr                       { print( "RPARENS" ); }
             rp:RPAREN                  { print( "RPARENS" ); }
           )
           )
  { System.out.print( "), " ); }
        ;
initDeclList
  { System.out.print( "(" ); print( "initDeclList" ); }
        :  (
                initDecl
                ( { print( "COMMA" ); } initDecl )*
           )
  { System.out.print( "), " ); }
        ;
initDecl
  { System.out.print( "(" ); print( "initDecl" ); String declName = ""; }
        :  (
                #(NInitDecl
                declarator
                ( attributeDecl )*
                ( a:ASSIGN              { print( a ); }
                  initializer
                | b:COLON               { print( b ); }
                  expr
                )?
                )
           )
  { System.out.print( "), " ); }
        ;
pointerGroup
  { System.out.print( "(" ); print( "pointerGroup" ); }
        :  (
                #( NPointerGroup
                   ( a:STAR             { print( a ); }
                    ( typeQualifier )*
                   )+
                )
           )
  { System.out.print( "), " ); }
        ;
idList
  { System.out.print( "(" ); print( "idList" ); }
        :  (
                i:ID                            { print( i ); }
                (  c:COMMA                      {  print( "COMMA" ); }
                   id:ID                        { print( id ); }
                )*
           )
  { System.out.print( "), " ); }
        ;
initializer
  { System.out.print( "(" ); print( "initializer" ); }
        :  (
                #( NInitializer (initializerElementLabel)? expr )
                | lcurlyInitializer
           )
  { System.out.print( "), " ); }
        ;
initializerElementLabel
  { System.out.print( "(" ); print( "initializerElementLabel" ); }
        :  (
            #( NInitializerElementLabel
                (
                    ( l:LBRACKET              { print( l ); }
                        expr
                        r:RBRACKET            { print( r ); }
                        (a1:ASSIGN             { print( a1 ); } )?
                    )
                    | i1:ID c:COLON           { print( i1 ); print( c ); }
                    | d:DOT i2:ID a2:ASSIGN      { print( d ); print( i2 ); print( a2 ); }
                )
            )
           )
  { System.out.print( "), " ); }
        ;
lcurlyInitializer
  { System.out.print( "(" ); print( "lcurlyInitializer" ); }
        :  (
              #(n:NLcurlyInitializer    { print( n ); tabs++; }
                initializerList
                rc:RCURLY               { tabs--; print( rc ); }
                )
           )
  { System.out.print( "), " ); }
        ;
initializerList
  { System.out.print( "(" ); print( "initializerList" ); }
        :  (
                ( i:initializer { commaSep( i ); }
                )*
           )
  { System.out.print( "), " ); }
        ;
declarator
  { System.out.print( "(" ); print( "declarator" ); }
        :  (
            #( NDeclarator
                ( pointerGroup )?
                ( id:ID                         {  currentOutput.print("(\"declarator.0\", "); print( id ); currentOutput.print("), ");  }
                | lp:LPAREN {  currentOutput.print("(\"declarator.1\", "); print( "LPARENS" ); currentOutput.print("), ");  } declarator rp:RPAREN {  currentOutput.print("(\"declarator.2\", "); print( "RPARENS" ); currentOutput.print("), ");  }
                )
                (   #( n:NParameterTypeList       { print( "LPARENS" );  /* print( n ); */ }
                    (
                        parameterTypeList
                        | (idList)?
                    )
                    r:RPAREN                      { print( "RPARENS" ); }
                    )
                 | lb:LBRACKET { print( lb ); } ( expr )? rb:RBRACKET { print( rb ); }
                )*
             )
           )
  { System.out.print( "), " ); }
        ;
parameterTypeList
  { System.out.print( "(" ); print( "parameterTypeList" ); }
        :  (
                ( parameterDeclaration
                    ( c:COMMA {  print( "COMMA" ); }
                      | s:SEMI { print( "SEMI" ); }
                    )?
                )+
                ( v:VARARGS { print( v ); } )?
           )
  { System.out.print( "), " ); }
        ;
parameterDeclaration
  { System.out.print( "(" ); print( "parameterDeclaration" ); }
        :  (
                #( NParameterDeclaration
                declSpecifiers
                (declarator | nonemptyAbstractDeclarator)?
                )
           )
  { System.out.print( "), " ); }
        ;
functionDef
  { System.out.print( "(" ); print( "functionDef WARNING" ); }
        :  (
            #( NFunctionDef
                ( functionDeclSpecifiers)?
                declarator
                (declaration
                 | v:VARARGS    { print( v ); }
                )*
                compoundStatement
            )
           )
  { System.out.print( "), " ); }
        ;
functionDeclSpecifiers
  { System.out.print( "(" ); print( "functionDeclSpecifiers WARNING" ); }
        :  (
         
                ( functionStorageClassSpecifier
                | typeQualifier
                | typeSpecifier
                )+
           )
  { System.out.print( "), " ); }
        ;
declarationList
  { System.out.print( "(" ); print( "declarationList WARNING" ); }
        :  (
         
                (   //ANTLR doesn't know that declarationList properly eats
                    //all the declarations so it warns about the ambiguity
                    options {
                        warnWhenFollowAmbig = false;
                    } :
                localLabelDecl
                |  declaration
                )+
           )
  { System.out.print( "), " ); }
        ;
localLabelDecl
  { System.out.print( "(" ); print( "localLabelDecl WARNING" ); }
        :  (
            #(a:"__label__"             { print( a ); }
              ( i:ID                    { commaSep( i ); }
              )+
                                        { print( "SEMI" ); }
            )
           )
  { System.out.print( "), " ); }
        ;
compoundStatement
  { System.out.print( "(" ); print( "compoundStatement WARNING" ); }
        :  (
                #( cs:NCompoundStatement                { print( cs ); tabs++; }
                ( declarationList
                | functionDef
                )*
                ( statementList )?
                rc:RCURLY                               { tabs--; print( rc ); }
                )
           )
  { System.out.print( "), " ); }
        ;
statementList
  { System.out.print( "(" ); print( "statementList WARNING" ); }
        :  (
                ( statement )+
           )
  { System.out.print( "), " ); }
        ;
statement
  { System.out.print( "(" ); print( "statement WARNING" ); }
        :  (
                statementBody
           )
  { System.out.print( "), " ); }
        ;
statementBody
  { System.out.print( "(" ); print( "statementBody WARNING" ); }
        :  (
                s:SEMI                          { print( "SEMI" ); }
        |       compoundStatement       // Group of statements
        |       #(NStatementExpr
                expr                    { print( "SEMI" ); }
                )                    // Expressions
// Iteration statements:
        |       #( w:"while" { print( w ); print( "LPARENS" ); }
                expr { print( "RPARENS" ); }
                statement )
        |       #( d:"do" { print( d ); }
                statement
                        { print( " while ( " ); }
                expr
                        { print( " );" ); }
                )
        |       #( f:"for" { print( f ); print( "LPARENS" ); }
                expr    { print( "SEMI" ); }
                expr    { print( "SEMI" ); }
                expr    { print( "RPARENS" ); }
                statement
                )
// Jump statements:
        |       #( g:"goto"             { print( g ); }
                   expr                 { print( "SEMI" ); }
                )
        |       c:"continue"            { print( c ); print( "SEMI" ); }
        |       b:"break"               { print( b ); print( "SEMI" ); }
        |       #( r:"return"           { print( r ); }
                ( expr )?
                                        { print( "SEMI" ); }
                )
// Labeled statements:
        |       #( NLabel
                ni:ID                   { print( ni ); print( ":" ); }
                ( statement )?
                )
        |       #(
                ca:"case"               { print( ca ); }
                expr                    { print( ":" ); }
                (statement)?
                )
        |       #(
                de:"default"            { print( de ); print( ":" ); }
                (statement)?
                )
// Selection statements:
        |       #( i:"if"               { print( i ); print( "LPARENS" ); }
                 expr                   { print( "RPARENS" ); }
                statement
                (   e:"else"            { print( e ); }
                    statement
                )?
                )
        |       #( sw:"switch"          { print( sw ); print( "LPARENS" ); }
                expr                    { print( "RPARENS" ); }
                statement
                )
           )
  { System.out.print( "), " ); }
        ;
expr
  { System.out.print( "(" ); print( "expr" ); }
        :  (
         
                binaryExpr
        |       conditionalExpr
        |       castExpr
        |       unaryExpr
        |       postfixExpr
        |       primaryExpr
        |       emptyExpr
        |       compoundStatementExpr
        |       initializer
        |       rangeExpr
        |       gnuAsmExpr
           )
  { System.out.print( "), " ); }
        ;
emptyExpr
  { System.out.print( "(" ); print( "emptyExpr" ); }
        :  (
            NEmptyExpression
           )
  { System.out.print( "), " ); }
        ;
compoundStatementExpr
  { System.out.print( "(" ); print( "compoundStatementExpr WARNING" ); }
        :  (
            #(l:LPAREN                  { print( "LPARENS" ); }
                compoundStatement
                r:RPAREN                { print( "RPARENS" ); }
            )
           )
  { System.out.print( "), " ); }
        ;
rangeExpr
  { System.out.print( "(" ); print( "rangeExpr" ); }
        :  (
            #(NRangeExpr expr v:VARARGS{ print( v ); } expr)
           )
  { System.out.print( "), " ); }
        ;
gnuAsmExpr
  { System.out.print( "(" ); print( "gnuAsmExpr WARNING" ); }
        :  (
            #(n:NGnuAsmExpr                        { print( n ); }
                (v:"volatile" { print( v ); } )?
                lp:LPAREN               { print( "LPARENS" ); }
                stringConst
                (  options { warnWhenFollowAmbig = false; }:
                    c1:COLON { print( c1 ); }
                    (strOptExprPair
                        ( c2:COMMA { print( "COMMA" ); } strOptExprPair)*
                    )?
                  (  options { warnWhenFollowAmbig = false; }:
                    c3:COLON            { print( c3 ); }
                      (strOptExprPair
                        ( c4:COMMA { print( "COMMA" ); } strOptExprPair)*
                      )?
                  )?
                )?
                ( c5:COLON              { print( c5 ); }
                  stringConst
                  ( c6:COMMA            { print( "COMMA" ); }
                    stringConst
                  )*
                )?
                rp:RPAREN               { print( "RPARENS" ); }
            )
           )
  { System.out.print( "), " ); }
        ;
strOptExprPair
  { System.out.print( "(" ); print( "strOptExprPair WARNING" ); }
        :  (
            stringConst
            (
            l:LPAREN                    { print( "LPARENS" ); }
            expr
            r:RPAREN                    { print( "RPARENS" ); }
            )?
           )
  { System.out.print( "), " ); }
        ;
binaryOperator
  { System.out.print( "(" ); print( "binaryOperator" ); }
        :  (
                ASSIGN
        |       DIV_ASSIGN
        |       PLUS_ASSIGN
        |       MINUS_ASSIGN
        |       STAR_ASSIGN
        |       MOD_ASSIGN
        |       RSHIFT_ASSIGN
        |       LSHIFT_ASSIGN
        |       BAND_ASSIGN
        |       BOR_ASSIGN
        |       BXOR_ASSIGN
        |       LOR
        |       LAND
        |       BOR
        |       BXOR
        |       BAND
        |       EQUAL
        |       NOT_EQUAL
        |       LT
        |       LTE
        |       GT
        |       GTE
        |       LSHIFT
        |       RSHIFT
        |       PLUS
        |       MINUS
        |       STAR
        |       DIV
        |       MOD
        |       NCommaExpr
           )
  { System.out.print( "), " ); }
        ;
binaryExpr
  { System.out.print( "(" ); print( "binaryExpr" ); }
        :  (
                b:binaryOperator
                    // no rules allowed as roots, so here I manually get
                    // the first and second children of the binary operator
                    // and then print them out in the right order
                                        {       TNode e1, e2;
                                                e1 = (TNode) b.getFirstChild();
                                                e2 = (TNode) e1.getNextSibling();
                                                expr( e1 );
                                                print( b );
                                                expr( e2 );
                                        }
           )
  { System.out.print( "), " ); }
        ;
conditionalExpr
  { System.out.print( "(" ); print( "conditionalExpr WARNING" ); }
        :  (
                #( q:QUESTION
                expr                    { print( q ); }
                ( expr )?
                c:COLON                 { print( c ); }
                expr
                )
           )
  { System.out.print( "), " ); }
        ;
castExpr
  { System.out.print( "(" ); print( "castExpr WARNING" ); }
        :  (
                #(
                c:NCast                 { print( c ); }
                typeName
                rp:RPAREN               { print( "RPARENS" ); }
                expr
                )
           )
  { System.out.print( "), " ); }
        ;
typeName
  { System.out.print( "(" ); print( "typeName" ); }
        :  (
                specifierQualifierList (nonemptyAbstractDeclarator)?
           )
  { System.out.print( "), " ); }
        ;
nonemptyAbstractDeclarator
  { System.out.print( "(" ); print( "nonemptyAbstractDeclarator" ); }
        :  (
            #( NNonemptyAbstractDeclarator
            (   pointerGroup
                (   (lp1:LPAREN                         { print( "LPARENS" ); }
                    (   nonemptyAbstractDeclarator
                        | parameterTypeList
                    )?
                    rp1:RPAREN                          { print( "RPARENS" ); }
                    )
                | (
                    lb1:LBRACKET                        { print( lb1 ); }
                    (expr)?
                    rb1:RBRACKET                        { print( rb1 ); }
                  )
                )*
            |  (   (lp2:LPAREN                          { print( "LPARENS" ); }
                    (   nonemptyAbstractDeclarator
                        | parameterTypeList
                    )?
                    rp2:RPAREN                          { print( "RPARENS" ); }
                   )
                | (
                    lb2:LBRACKET                        { print( lb2 ); }
                    (expr)?
                    rb2:RBRACKET                        { print( rb2 ); }
                  )
                )+
            )
            )
           )
  { System.out.print( "), " ); }
        ;
unaryExpr
  { System.out.print( "(" ); print( "unaryExpr" ); }
        :  (
                #( i:INC { print( i ); } expr )
        |       #( d:DEC { print( d ); } expr )
        |       #( NUnaryExpr u:unaryOperator { print( u ); } expr)
        |       #( s:"sizeof"                           { print( s ); }
                    ( ( LPAREN typeName )=>
                        lps:LPAREN                      { print( "LPARENS" ); }
                        typeName
                        rps:RPAREN                      { print( "RPARENS" ); }
                    | expr
                    )
                )
        |       #( a:"__alignof"                             { print( a ); }
                    ( ( LPAREN typeName )=>
                        lpa:LPAREN                      { print( "LPARENS" ); }
                        typeName
                        rpa:RPAREN                      { print( "RPARENS" ); }
                    | expr
                    )
                )
           )
  { System.out.print( "), " ); }
        ;
unaryOperator
  { System.out.print( "(" ); print( "unaryOperator" ); }
        :  (
                BAND
        |       STAR
        |       PLUS
        |       MINUS
        |       BNOT
        |       LNOT
        |       LAND
        |       "__real"
        |       "__imag"
           )
  { System.out.print( "), " ); }
        ;
postfixExpr
  { System.out.print( "(" ); print( "postfixExpr" ); }
        :  (
                #( NPostfixExpr
                    primaryExpr
                    ( a:PTR b:ID                                { print( a ); print( b ); }
                    | c:DOT d:ID                                { print( c ); print( d ); }
                    | #( n:NFunctionCallArgs                          { print( n ); }
                        (argExprList)?
                        rp:RPAREN                                     { print( "RPARENS" ); }
                        )
                    | lb:LBRACKET                               { print( lb ); }
                        expr
                        rb:RBRACKET                             { print( rb ); }
                    | f:INC                                     { print( f ); }
                    | g:DEC                                     { print( g ); }
                    )+
                )
           )
  { System.out.print( "), " ); }
        ;
primaryExpr
  { System.out.print( "(" ); print( "primaryExpr" ); }
        :  (
                i:ID                            { print( i ); }
        |       n:Number                        { print( n ); }
        |       charConst
        |       stringConst
// JTC:
// ID should catch the enumerator
// leaving it in gives ambiguous err
//      | enumerator
        |       #( eg:NExpressionGroup          { print( eg ); }
                 expr                           { print( "RPARENS" ); }
                )
           )
  { System.out.print( "), " ); }
        ;
argExprList
  { System.out.print( "(" ); print( "argExprList WARNING" ); }
        :  (
                expr ( { print( "COMMA" ); } expr )*
           )
  { System.out.print( "), " ); }
        ;
protected
charConst
  { System.out.print( "(" ); print( "charConst" ); }
        :  (
                c:CharLiteral                   { print( c ); }
           )
  { System.out.print( "), " ); }
        ;
protected
stringConst
  { System.out.print( "(" ); print( "stringConst" ); }
        :  (
                #( NStringSeq
                    (
                    s:StringLiteral                 { print( s ); }
                    )+
                )
           )
  { System.out.print( "), " ); }
        ;
protected
intConst
  { System.out.print( "(" ); print( "intConst" ); }
        :  (
                IntOctalConst
        |       LongOctalConst
        |       UnsignedOctalConst
        |       IntIntConst
        |       LongIntConst
        |       UnsignedIntConst
        |       IntHexConst
        |       LongHexConst
        |       UnsignedHexConst
           )
  { System.out.print( "), " ); }
        ;
protected
floatConst
  { System.out.print( "(" ); print( "floatConst" ); }
        :  (
                FloatDoubleConst
        |       DoubleDoubleConst
        |       LongDoubleConst
           )
  { System.out.print( "), " ); }
        ;
