typedef unsigned int size_t;
typedef long long int64;
typedef unsigned long long uint64;
typedef unsigned char uchar;

typedef void CvArr;

#define IPL_DEPTH_SIGN 0x80000000

#define IPL_DEPTH_1U     1
#define IPL_DEPTH_8U     8
#define IPL_DEPTH_16U   16
#define IPL_DEPTH_32F   32

#define IPL_DEPTH_8S  (IPL_DEPTH_SIGN| 8)
#define IPL_DEPTH_16S (IPL_DEPTH_SIGN|16)
#define IPL_DEPTH_32S (IPL_DEPTH_SIGN|32)

#define IPL_DATA_ORDER_PIXEL  0
#define IPL_DATA_ORDER_PLANE  1

#define IPL_ORIGIN_TL 0
#define IPL_ORIGIN_BL 1

#define IPL_ALIGN_4BYTES   4
#define IPL_ALIGN_8BYTES   8
#define IPL_ALIGN_16BYTES 16
#define IPL_ALIGN_32BYTES 32

#define IPL_ALIGN_DWORD   IPL_ALIGN_4BYTES
#define IPL_ALIGN_QWORD   IPL_ALIGN_8BYTES

typedef struct _IplImage {
    int  nSize;
    int  ID;
    int  nChannels;
    int  depth;
    int  dataOrder;
    int  origin;
    int  width;
    int  height;
    struct _IplROI *roi;
    int  imageSize;
    char *imageData;
    int  widthStep;
    char *imageDataOrigin;
}
IplImage;

typedef struct _IplTileInfo IplTileInfo;

typedef struct _IplROI {
    int  coi;
    int  xOffset;
    int  yOffset;
    int  width;
    int  height;
}
IplROI;

typedef struct _IplConvKernel
{
    int  nCols;
    int  nRows;
    int  anchorX;
    int  anchorY;
    int *values;
    int  nShiftR;
}
IplConvKernel;

typedef struct _IplConvKernelFP
{
    int  nCols;
    int  nRows;
    int  anchorX;
    int  anchorY;
    float *values;
}
IplConvKernelFP;

#define IPL_IMAGE_HEADER 1
#define IPL_IMAGE_DATA   2
#define IPL_IMAGE_ROI    4

#define IPL_IMAGE_MAGIC_VAL  ((int)sizeof(IplImage))


#define IPL_IMAGE_FILE_MAGIC_VAL  112

#define CV_IS_IMAGE_HDR(img)     ((img) != NULL && ((const IplImage*)(img))->nSize == sizeof(IplImage))

#define CV_IS_IMAGE(img)     (CV_IS_IMAGE_HDR(img) && ((IplImage*)img)->imageData != NULL)

#define IPL_DEPTH_64F  64


#define CV_IMAGE_ELEM( image, elemtype, row, col )     (((elemtype*)((image)->imageData + (image)->widthStep*(row))[(col)]))




#define CV_8U   0
#define CV_8S   1
#define CV_16S  2
#define CV_32S  3
#define CV_32F  4
#define CV_64F  5
#define CV_USRTYPE1 6
#define CV_USRTYPE2 7

#define CV_8UC1 (CV_8U + 0*8)
#define CV_8UC2 (CV_8U + 1*8)
#define CV_8UC3 (CV_8U + 2*8)
#define CV_8UC4 (CV_8U + 3*8)

#define CV_8SC1 (CV_8S + 0*8)
#define CV_8SC2 (CV_8S + 1*8)
#define CV_8SC3 (CV_8S + 2*8)
#define CV_8SC4 (CV_8S + 3*8)

#define CV_16SC1 (CV_16S + 0*8)
#define CV_16SC2 (CV_16S + 1*8)
#define CV_16SC3 (CV_16S + 2*8)
#define CV_16SC4 (CV_16S + 3*8)

#define CV_32SC1 (CV_32S + 0*8)
#define CV_32SC2 (CV_32S + 1*8)
#define CV_32SC3 (CV_32S + 2*8)
#define CV_32SC4 (CV_32S + 3*8)

#define CV_32FC1 (CV_32F + 0*8)
#define CV_32FC2 (CV_32F + 1*8)
#define CV_32FC3 (CV_32F + 2*8)
#define CV_32FC4 (CV_32F + 3*8)

#define CV_64FC1 (CV_64F + 0*8)
#define CV_64FC2 (CV_64F + 1*8)
#define CV_64FC3 (CV_64F + 2*8)
#define CV_64FC4 (CV_64F + 3*8)

#define CV_AUTO_STEP  0x7fffffff
#define CV_WHOLE_ARR  cvSlice( 0, 0x3fffffff )

#define CV_MAT_CN_MASK          (3 << 3)
#define CV_MAT_CN(flags)        ((((flags) & CV_MAT_CN_MASK) >> 3) + 1)
#define CV_MAT_DEPTH_MASK       7
#define CV_MAT_DEPTH(flags)     ((flags) & CV_MAT_DEPTH_MASK)
#define CV_MAT_TYPE_MASK        31
#define CV_MAT_TYPE(flags)      ((flags) & CV_MAT_TYPE_MASK)
#define CV_MAT_FMT_MASK         511
#define CV_MAT_FMT(flags)       ((flags) & CV_MAT_FMT_MASK)
#define CV_MAT_CONT_FLAG_SHIFT  9
#define CV_MAT_CONT_FLAG        (1 << CV_MAT_CONT_FLAG_SHIFT)
#define CV_IS_MAT_CONT(flags)   ((flags) & CV_MAT_CONT_FLAG)
#define CV_IS_CONT_MAT          CV_IS_MAT_CONT
#define CV_MAT_TEMP_FLAG_SHIFT  10
#define CV_MAT_TEMP_FLAG        (1 << CV_MAT_TEMP_FLAG_SHIFT)
#define CV_IS_TEMP_MAT(flags)   ((flags) & CV_MAT_TEMP_FLAG)

#define CV_MAGIC_MASK       0xFFFF0000
#define CV_MAT_MAGIC_VAL    0x42420000

typedef struct CvMat
{
    int type;
    int step;

    int* refcount;

    union
    {
        uchar* ptr;
        short* s;
        int* i;
        float* fl;
        double* db;
    } data;

    int rows;
    int cols;

} CvMat;


#define CV_IS_MAT_HDR(mat)     ((mat) != NULL && (((const CvMat*)(mat))->type & CV_MAGIC_MASK) == CV_MAT_MAGIC_VAL)

#define CV_IS_MAT(mat)     (CV_IS_MAT_HDR(mat) && ((const CvMat*)(mat))->data.ptr != NULL)

#define CV_IS_MASK_ARR(mat)     (((mat)->type & (CV_MAT_TYPE_MASK & ~CV_8SC1)) == 0)

#define CV_ARE_TYPES_EQ(mat1, mat2)     ((((mat1)->type ^ (mat2)->type) & CV_MAT_TYPE_MASK) == 0)

#define CV_ARE_CNS_EQ(mat1, mat2)     ((((mat1)->type ^ (mat2)->type) & CV_MAT_CN_MASK) == 0)

#define CV_ARE_DEPTHS_EQ(mat1, mat2)     ((((mat1)->type ^ (mat2)->type) & CV_MAT_DEPTH_MASK) == 0)

#define CV_ARE_SIZES_EQ(mat1, mat2)     ((mat1)->height == (mat2)->height && (mat1)->width == (mat2)->width)

#define CV_IS_MAT_CONST(mat)     (((mat)->height|(mat)->width) == 1)

#define CV_ELEM_SIZE(type)     (CV_MAT_CN(type) << ((0xe90 >> CV_MAT_DEPTH(type)*2) & 3))


CvMat cvMat( int rows, int cols, int type, void* data );

#define CV_MAT_ELEM_PTR_FAST( mat, row, col, pix_size )     (assert( (unsigned)(row) < (unsigned)(mat).rows &&              (unsigned)(col) < (unsigned)(mat).cols ),      (mat).data.ptr + (size_t)(mat).step*(row) + (pix_size)*(col))

#define CV_MAT_ELEM_PTR( mat, row, col )     CV_MAT_ELEM_PTR_FAST( mat, row, col, CV_ELEM_SIZE((mat).type) )

#define CV_MAT_ELEM( mat, elemtype, row, col )     (*(elemtype*)CV_MAT_ELEM_PTR_FAST( mat, row, col, sizeof(elemtype)))


double  cvmGet( const CvMat* mat, int i, int j );

void  cvmSet( CvMat* mat, int i, int j, double val );

#define CV_MATND_MAGIC_VAL    0x42430000
#define CV_MAX_DIM 16

#define CV_MAT_LIKE_FLAG_SHIFT  11
#define CV_MAT_LIKE_FLAG        (1 << CV_MAT_LIKE_FLAG_SHIFT)

typedef struct CvMatND
{
    int type;
    int dims;

    int* refcount;
    union
    {
        uchar* ptr;
        float* fl;
        double* db;
        int* i;
        short* s;
    } data;

    struct
    {
        int size;
        int step;
    }
    dim[CV_MAX_DIM];
}
CvMatND;

#define CV_IS_MATND_HDR(mat)     ((mat) != NULL && (((const CvMatND*)(mat))->type & CV_MAGIC_MASK) == CV_MATND_MAGIC_VAL)

#define CV_IS_MATND(mat)     (CV_IS_MATND_HDR(mat) && ((const CvMatND*)(mat))->data.ptr != NULL)




#define CV_SPARSE_MAT_MAGIC_VAL    0x42440000

struct CvSet;

typedef struct CvSparseMat
{
    int type;
    int dims;
    int* refcount;
    struct CvSet* heap;
    void** hashtable;
    int hashsize;
    int total;
    int valoffset;
    int idxoffset;
    int size[CV_MAX_DIM];
}
CvSparseMat;

#define CV_IS_SPARSE_MAT_HDR(mat)     ((mat) != NULL &&     (((const CvSparseMat*)(mat))->type & CV_MAGIC_MASK) == CV_SPARSE_MAT_MAGIC_VAL)

#define CV_IS_SPARSE_MAT(mat)     CV_IS_SPARSE_MAT_HDR(mat)



typedef struct CvSparseNode
{
    unsigned hashval;
    struct CvSparseNode* next;
}
CvSparseNode;

typedef struct CvSparseMatIterator
{
    CvSparseMat* mat;
    CvSparseNode* node;
    int curidx;
}
CvSparseMatIterator;

#define CV_NODE_VAL(mat,node)   ((void*)((uchar*)(node) + (mat)->valoffset))
#define CV_NODE_IDX(mat,node)   ((int*)((uchar*)(node) + (mat)->idxoffset))



typedef int CvHistType;

#define CV_HIST_MAGIC_VAL     0x42450000
#define CV_HIST_UNIFORM_FLAG  (1 << 10)


#define CV_HIST_RANGES_FLAG   (1 << 11)

#define CV_HIST_ARRAY         0
#define CV_HIST_SPARSE        1
#define CV_HIST_TREE          CV_HIST_SPARSE

#define CV_HIST_UNIFORM       1

typedef struct CvHistogram
{
    int     type;
    CvArr*  bins;
    float   thresh[CV_MAX_DIM][2];
    float** thresh2;
    CvMatND mat;
}
CvHistogram;

#define CV_IS_HIST( hist )     ((hist) != NULL  &&      (((CvHistogram*)(hist))->type & CV_MAGIC_MASK) == CV_HIST_MAGIC_VAL &&      (hist)->bins != NULL)

#define CV_IS_UNIFORM_HIST( hist )     (((hist)->type & CV_HIST_UNIFORM_FLAG) != 0)

#define CV_IS_SPARSE_HIST( hist )     CV_IS_SPARSE_MAT((hist)->bins)

#define CV_HIST_HAS_RANGES( hist )     (((hist)->type & CV_HIST_RANGES_FLAG) != 0)




int  cvRound( double val );
int  cvFloor( double val );
int  cvCeil( double val );

typedef struct CvRect
{
    int x;
    int y;
    int width;
    int height;
}
CvRect;

CvRect  cvRect( int x, int y, int width, int height );
IplROI  cvRectToROI( CvRect rect, int coi );
CvRect  cvROIToRect( IplROI roi );
CvRect  cvROIToRect( IplROI roi );

#define CV_TERMCRIT_ITER    1
#define CV_TERMCRIT_NUMB    CV_TERMCRIT_ITER
#define CV_TERMCRIT_EPS     2

typedef struct CvTermCriteria
{
    int    type;
    int    maxIter;
    double epsilon;
}
CvTermCriteria;

CvTermCriteria  cvTermCriteria( int type, int maxIter, double epsilon );

typedef struct CvPoint
{
    int x;
    int y;
}
CvPoint;


CvPoint  cvPoint( int x, int y );

typedef struct CvPoint2D32f
{
    float x;
    float y;
}
CvPoint2D32f;


CvPoint2D32f  cvPoint2D32f( double x, double y );
CvPoint2D32f  cvPointTo32f( CvPoint pt );
CvPoint  cvPointFrom32f( CvPoint2D32f pt );
CvPoint  cvPointFrom32f( CvPoint2D32f pt );

typedef struct CvPoint3D32f
{
    float x;
    float y;
    float z;
}
CvPoint3D32f;


CvPoint3D32f  cvPoint3D32f( double x, double y, double z );

typedef struct CvPoint2D64d
{
    double x;
    double y;
}
CvPoint2D64d;


typedef struct CvPoint3D64d
{
    double x;
    double y;
    double z;
}
CvPoint3D64d;




typedef struct
{
    int width;
    int height;
}
CvSize;

CvSize  cvSize( int width, int height );

typedef struct CvSize2D32f
{
    float width;
    float height;
}
CvSize2D32f;


CvSize2D32f  cvSize2D32f( double width, double height );

typedef struct CvBox2D
{
    CvPoint2D32f center;
    CvSize2D32f  size;
    float angle;
}
CvBox2D;



typedef struct CvSlice
{
    int  startIndex, endIndex;
} CvSlice;

CvSlice  cvSlice( int start, int end );

#define CV_WHOLE_SEQ  cvSlice(0, 0x3fffffff)




typedef struct CvScalar
{
    double val[4];
}
CvScalar;


CvScalar  cvScalar( double a, double b ,
                               double c , double d );

CvScalar  cvRealScalar( double a );
CvScalar  cvScalarAll( double a );

struct CvSeq;

typedef struct CvConnectedComp
{
    double area;
    double value;
    CvRect rect;
    struct CvSeq* contour;
}
CvConnectedComp;




#define CV_PI   3.1415926535897932384626433832795

#define CV_SWAP(a,b,t) ((t) = (a), (a) = (b), (b) = (t))


#define  CV_IMIN(a, b)  ((a) ^ (((a)^(b)) & (((a) < (b)) - 1)))

#define  CV_IMAX(a, b)  ((a) ^ (((a)^(b)) & (((a) > (b)) - 1)))


#define  CV_IABS(a)     (((a) ^ ((a) < 0 ? -1 : 0)) - ((a) < 0 ? -1 : 0))
#define  CV_SIGN(a)     (((a) < 0 ? -1 : 0) | ((a) > 0))


#define  CV_INIT_3X3_DELTAS( deltas, step, nch )     ((deltas)[0] =  (nch),  (deltas)[1] = -(step) + (nch),      (deltas)[2] = -(step), (deltas)[3] = -(step) - (nch),      (deltas)[4] = -(nch),  (deltas)[5] =  (step) - (nch),      (deltas)[6] =  (step), (deltas)[7] =  (step) + (nch))


#define CV_SQRT_MAGIC  0xbe6f0000

float  cvInvSqrt( float arg );
float  cvSqrt( float arg );

typedef enum CvDisType
{
    CV_DIST_USER         = -1,
    CV_DIST_L1           = 1,
    CV_DIST_L2           = 2,
    CV_DIST_C            = 3,
    CV_DIST_L12          = 4,
    CV_DIST_FAIR         = 5,
    CV_DIST_WELSCH       = 6,
    CV_DIST_HUBER        = 7
}
CvDisType;



typedef enum CvFilter
{
    CV_GAUSSIAN_5x5 = 7
}
CvFilter;







typedef struct CvMemBlock
{
    struct CvMemBlock*  prev;
    struct CvMemBlock*  next;
}
CvMemBlock;

#define CV_STORAGE_MAGIC_VAL    0x42890000

typedef struct CvMemStorage
{
    int     signature;
    CvMemBlock* bottom;
    CvMemBlock* top;
    struct  CvMemStorage* parent;
    int     block_size;
    int     free_space;
}
CvMemStorage;

#define CV_IS_STORAGE(storage)     ((storage) != NULL &&     (((CvMemStorage*)(storage))->signature & CV_MAGIC_MASK) == CV_STORAGE_MAGIC_VAL)


typedef struct CvMemStoragePos
{
    CvMemBlock* top;
    int  free_space;
}
CvMemStoragePos;




typedef struct CvSeqBlock
{
    struct CvSeqBlock*  prev;
    struct CvSeqBlock*  next;
    int    start_index;
    int    count;
    char*  data;}
CvSeqBlock;

typedef struct CvSeq
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
}
CvSeq;

typedef struct CvSetElem
{
/*  CV_SET_ELEM_FIELDS(CvSetElem) */
    int  flags;
    struct CvSetElem* next_free;
}
CvSetElem;

typedef struct CvSet
{
/*  CV_SET_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
}
CvSet;


#define CV_SET_ELEM_IDX_MASK   ((1 << 24) - 1)
#define CV_SET_ELEM_FREE_FLAG  (1 << (sizeof(int)*8-1))


#define CV_IS_SET_ELEM( ptr )  (((CvSetElem*)(ptr))->flags >= 0)

typedef struct CvGraphEdge
{
/*  CV_GRAPH_EDGE_FIELDS() */
    int flags;
    float weight;
    struct CvGraphEdge* next[2];
    struct CvGraphVtx* vtx[2];
}
CvGraphEdge;

typedef struct CvGraphVtx
{
/*  CV_GRAPH_VERTEX_FIELDS() */
    int flags;
    struct CvGraphEdge* first;
}
CvGraphVtx;

typedef struct CvGraphVtx2D
{
/*  CV_GRAPH_VERTEX_FIELDS() */
    int flags;
    struct CvGraphEdge* first;
    CvPoint2D32f* ptr;
}
CvGraphVtx2D;

typedef struct CvGraph
{
/*  CV_GRAPH_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
    CvSet* edges;
}
CvGraph;



typedef struct CvChain
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvPoint  origin;
}
CvChain;

typedef struct CvContour
{
/*  CV_CONTOUR_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvRect rect;
    int color;
    int reserved[3];
}
CvContour;

typedef CvContour CvPoint2DSeq;



#define CV_SEQ_MAGIC_VAL             0x42990000
#define CV_IS_SEQ(seq)     ((seq) != NULL && (((CvSeq*)(seq))->flags & CV_MAGIC_MASK) == CV_SEQ_MAGIC_VAL)

#define CV_SET_MAGIC_VAL             0x42980000
#define CV_IS_SET(set)     ((set) != NULL && (((CvSeq*)(set))->flags & CV_MAGIC_MASK) == CV_SET_MAGIC_VAL)

#define CV_SEQ_ELTYPE_BITS           5
#define CV_SEQ_ELTYPE_MASK           ((1 << CV_SEQ_ELTYPE_BITS) - 1)

#define CV_SEQ_ELTYPE_POINT          CV_32SC2
#define CV_SEQ_ELTYPE_CODE           CV_8UC1
#define CV_SEQ_ELTYPE_GENERIC        0
#define CV_SEQ_ELTYPE_PTR            CV_USRTYPE1
#define CV_SEQ_ELTYPE_PPOINT         CV_SEQ_ELTYPE_PTR
#define CV_SEQ_ELTYPE_INDEX          CV_32SC1
#define CV_SEQ_ELTYPE_GRAPH_EDGE     0
#define CV_SEQ_ELTYPE_GRAPH_VERTEX   0
#define CV_SEQ_ELTYPE_TRIAN_ATR      0
#define CV_SEQ_ELTYPE_CONNECTED_COMP 0
#define CV_SEQ_ELTYPE_POINT3D        CV_32FC3

#define CV_SEQ_KIND_BITS        5
#define CV_SEQ_KIND_MASK        (((1 << CV_SEQ_KIND_BITS) - 1)<<CV_SEQ_ELTYPE_BITS)


#define CV_SEQ_KIND_GENERIC     (0 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_CURVE       (1 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_BIN_TREE    (2 << CV_SEQ_ELTYPE_BITS)


#define CV_SEQ_KIND_GRAPH       (3 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_SUBDIV2D    (4 << CV_SEQ_ELTYPE_BITS)

#define CV_SEQ_FLAG_SHIFT       (CV_SEQ_KIND_BITS + CV_SEQ_ELTYPE_BITS)


#define CV_SEQ_FLAG_CLOSED     (1 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_SIMPLE     (2 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_CONVEX     (4 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_HOLE       (8 << CV_SEQ_FLAG_SHIFT)


#define CV_GRAPH_FLAG_ORIENTED (1 << CV_SEQ_FLAG_SHIFT)

#define CV_GRAPH               CV_SEQ_KIND_GRAPH
#define CV_ORIENTED_GRAPH      (CV_SEQ_KIND_GRAPH|CV_GRAPH_FLAG_ORIENTED)


#define CV_SEQ_POINT_SET       (CV_SEQ_KIND_GENERIC| CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POINT3D_SET     (CV_SEQ_KIND_GENERIC| CV_SEQ_ELTYPE_POINT3D)
#define CV_SEQ_POLYLINE        (CV_SEQ_KIND_CURVE  | CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POLYGON         (CV_SEQ_FLAG_CLOSED | CV_SEQ_POLYLINE )
#define CV_SEQ_CONTOUR         CV_SEQ_POLYGON
#define CV_SEQ_SIMPLE_POLYGON  (CV_SEQ_FLAG_SIMPLE | CV_SEQ_POLYGON  )


#define CV_SEQ_CHAIN           (CV_SEQ_KIND_CURVE  | CV_SEQ_ELTYPE_CODE)
#define CV_SEQ_CHAIN_CONTOUR   (CV_SEQ_FLAG_CLOSED | CV_SEQ_CHAIN)


#define CV_SEQ_POLYGON_TREE    (CV_SEQ_KIND_BIN_TREE  | CV_SEQ_ELTYPE_TRIAN_ATR)


#define CV_SEQ_CONNECTED_COMP  (CV_SEQ_KIND_GENERIC  | CV_SEQ_ELTYPE_CONNECTED_COMP)


#define CV_SEQ_INDEX           (CV_SEQ_KIND_GENERIC  | CV_SEQ_ELTYPE_INDEX)

#define CV_SEQ_ELTYPE( seq )   ((seq)->flags & CV_SEQ_ELTYPE_MASK)
#define CV_SEQ_KIND( seq )     ((seq)->flags & CV_SEQ_KIND_MASK )


#define CV_IS_SEQ_INDEX( seq )      ((CV_SEQ_ELTYPE(seq) == CV_SEQ_ELTYPE_INDEX) &&                                      (CV_SEQ_KIND(seq) == CV_SEQ_KIND_GENERIC))

#define CV_IS_SEQ_CURVE( seq )      (CV_SEQ_KIND(seq) == CV_SEQ_KIND_CURVE)
#define CV_IS_SEQ_CLOSED( seq )     (((seq)->flags & CV_SEQ_FLAG_CLOSED) != 0)
#define CV_IS_SEQ_CONVEX( seq )     (((seq)->flags & CV_SEQ_FLAG_CONVEX) != 0)
#define CV_IS_SEQ_HOLE( seq )       (((seq)->flags & CV_SEQ_FLAG_HOLE) != 0)
#define CV_IS_SEQ_SIMPLE( seq )     ((((seq)->flags & CV_SEQ_FLAG_SIMPLE) != 0) ||                                     CV_IS_SEQ_CONVEX(seq))


#define CV_IS_SEQ_POINT_SET( seq )     ((CV_SEQ_ELTYPE(seq) == CV_32SC2 || CV_SEQ_ELTYPE(seq) == CV_32FC2))

#define CV_IS_SEQ_POINT_SUBSET( seq )     (CV_IS_SEQ_INDEX( seq ) || CV_SEQ_ELTYPE(seq) == CV_SEQ_ELTYPE_PPOINT)

#define CV_IS_SEQ_POLYLINE( seq )     (CV_SEQ_KIND(seq) == CV_SEQ_KIND_CURVE && CV_IS_SEQ_POINT_SET(seq))

#define CV_IS_SEQ_POLYGON( seq )     (CV_IS_SEQ_POLYLINE(seq) && CV_IS_SEQ_CLOSED(seq))

#define CV_IS_SEQ_CHAIN( seq )     (CV_SEQ_KIND(seq) == CV_SEQ_KIND_CURVE && (seq)->elem_size == 1)

#define CV_IS_SEQ_CONTOUR( seq )     (CV_IS_SEQ_CLOSED(seq) && (CV_IS_SEQ_POLYLINE(seq) || CV_IS_SEQ_CHAIN(seq)))

#define CV_IS_SEQ_CHAIN_CONTOUR( seq )     (CV_IS_SEQ_CHAIN( seq ) && CV_IS_SEQ_CLOSED( seq ))

#define CV_IS_SEQ_POLYGON_TREE( seq )     (CV_SEQ_ELTYPE (seq) ==  CV_SEQ_ELTYPE_TRIAN_ATR &&     CV_SEQ_KIND( seq ) ==  CV_SEQ_KIND_BIN_TREE )

#define CV_IS_GRAPH( seq )     (CV_IS_SET(seq) && CV_SEQ_KIND((CvSet*)(seq)) == CV_SEQ_KIND_GRAPH)

#define CV_IS_GRAPH_ORIENTED( seq )     (((seq)->flags & CV_GRAPH_FLAG_ORIENTED) != 0)

#define CV_IS_SUBDIV2D( seq )     (CV_IS_SET(seq) && CV_SEQ_KIND((CvSet*)(seq)) == CV_SEQ_KIND_SUBDIV2D)

typedef struct CvSeqWriter
{
/*  CV_SEQ_WRITER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int  reserved[4];
}
CvSeqWriter;

typedef struct CvSeqReader
{
/*  CV_SEQ_READER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int          delta_index;
    char*        prev_elem;
    int  reserved[4];
}
CvSeqReader;

#define  CV_GET_SEQ_ELEM( elem_type, seq, index )  (   assert(sizeof((seq)->first[0]) == sizeof(CvSeqBlock) &&     (seq)->elem_size == sizeof(elem_type)),     (elem_type*)((seq)->first && (unsigned)index <     (unsigned)((seq)->first->count) ?     (seq)->first->data + (index) * sizeof(elem_type) :     cvGetSeqElem( (CvSeq*)(seq), (index), NULL )))



#define CV_WRITE_SEQ_ELEM_VAR( elem_ptr, writer ) {     if( (writer).ptr >= (writer).block_max )     {         cvCreateSeqBlock( &writer);     }     memcpy((writer).ptr, elem_ptr, (writer).seq->elem_size);     (writer).ptr += (writer).seq->elem_size; }

#define CV_WRITE_SEQ_ELEM( elem, writer ) {     assert( (writer).seq->elem_size == sizeof(elem));     if( (writer).ptr >= (writer).block_max )     {         cvCreateSeqBlock( &writer);     }     assert( (writer).ptr <= (writer).block_max - sizeof(elem));     memcpy((writer).ptr, &elem, sizeof(elem));     (writer).ptr += sizeof(elem); }



#define CV_NEXT_SEQ_ELEM( elem_size, reader ) {     if( ((reader).ptr += (elem_size)) >= (reader).block_max )     {         cvChangeSeqBlock( &(reader), 1 );     } }



#define CV_PREV_SEQ_ELEM( elem_size, reader ) {     if( ((reader).ptr -= (elem_size)) < (reader).block_min )     {         cvChangeSeqBlock( &(reader), -1 );     } }


#define CV_READ_SEQ_ELEM( elem, reader ) {     assert( (reader).seq->elem_size == sizeof(elem));     memcpy( &(elem), (reader).ptr, sizeof((elem)));     CV_NEXT_SEQ_ELEM( sizeof(elem), reader ) }


#define CV_REV_READ_SEQ_ELEM( elem, reader ) {     assert( (reader).seq->elem_size == sizeof(elem));     memcpy(&(elem), (reader).ptr, sizeof((elem)));     CV_PREV_SEQ_ELEM( sizeof(elem), reader ) }


#define CV_READ_CHAIN_POINT( _pt, reader ) {     (_pt) = (reader).pt;     if( (reader).ptr )     {         CV_READ_SEQ_ELEM( (reader).code, (*((CvSeqReader*)&(reader))));         assert( ((reader).code & ~7) == 0 );         (reader).pt.x += (reader).deltas[(reader).code][0];         (reader).pt.y += (reader).deltas[(reader).code][1];     } }

#define CV_CURRENT_POINT( reader )  (*((CvPoint*)((reader).ptr)))
#define CV_PREV_POINT( reader )     (*((CvPoint*)((reader).prev_elem)))

#define CV_READ_EDGE( pt1, pt2, reader ) {     assert( sizeof(pt1) == sizeof(CvPoint) &&             sizeof(pt2) == sizeof(CvPoint) &&             reader.seq->elem_size == sizeof(CvPoint));     (pt1) = CV_PREV_POINT( reader );     (pt2) = CV_CURRENT_POINT( reader );     (reader).prev_elem = (reader).ptr;     CV_NEXT_SEQ_ELEM( sizeof(CvPoint), (reader)); }




#define  CV_NEXT_GRAPH_EDGE( edge, vertex )      (assert((edge)->vtx[0] == (vertex) || (edge)->vtx[1] == (vertex)),       (edge)->next[(edge)->vtx[1] == (vertex)])




typedef struct
{
    const char* file;
    int         line;
}
CvStackRecord;






typedef float*   CvVect32f;
typedef float*   CvMatr32f;
typedef double*  CvVect64d;
typedef double*  CvMatr64d;

typedef struct CvMatrix3
{
    float m[3][3];
}
CvMatrix3;

typedef enum CvStatus
{
    CV_INPLACE_NOT_SUPPORTED_ERR= -112,
    CV_UNMATCHED_ROI_ERR        = -111,
    CV_NOTFOUND_ERR             = -110,
    CV_BADCONVERGENCE_ERR       = -109,

    CV_BADDEPTH_ERR             = -107,
    CV_BADROI_ERR               = -106,
    CV_BADHEADER_ERR            = -105,
    CV_UNMATCHED_FORMATS_ERR    = -104,
    CV_UNSUPPORTED_COI_ERR      = -103,
    CV_UNSUPPORTED_CHANNELS_ERR = -102,
    CV_UNSUPPORTED_DEPTH_ERR    = -101,
    CV_UNSUPPORTED_FORMAT_ERR   = -100,


    CV_BADARG_ERR      = -49,
    CV_NOTDEFINED_ERR  = -48,

    CV_BADCHANNELS_ERR = -47,
    CV_BADRANGE_ERR    = -44,
    CV_BADSTEP_ERR     = -29,

    CV_BADFLAG_ERR     =  -12,
    CV_DIV_BY_ZERO_ERR =  -11,
    CV_BADCOEF_ERR     =  -10,

    CV_BADFACTOR_ERR   =  -7,
    CV_BADPOINT_ERR    =  -6,
    CV_BADSCALE_ERR    =  -4,
    CV_OUTOFMEM_ERR    =  -3,
    CV_NULLPTR_ERR     =  -2,
    CV_BADSIZE_ERR     =  -1,
    CV_NO_ERR          =   0,
    CV_OK              =   CV_NO_ERR
}
CvStatus;
