#! /usr/bin/env python
from __future__ import generators

import sys
from types import *

""" Reformats a ".data" file for convenience of man and machine

Also some sequences of tokens are combined into strings. For example,
"int" and "*" will be combined into "int *".
"""
# These are a pair of magic tokens. See "insert_prints2.py".
TLMB = 'TLMB827'
TLME = 'TLME561'
# Other operators and separators can be added here if that is useful.
special_dict = { 'SEMI' : ';', 'COMMA' : ',', 'LPARENS' : '(',
                 'RPARENS' : ')' }
special_tokens = ('SEMI', 'COMMA', 'LPARENS', 'RPARENS')
stop_tokens = ('SEMI', 'COMMA', 'LPARENS', 'RPARENS', '{', '}', 'const')

# Combine a list of words into one string.
#      ('aaa', 'bbb') -> ' "aaa bbb", '
# Function "walk" accumulates a list of words. When it decides that the list
# of words is long enough, it calls "CombineWords" and resets words to [].
def CombineWords(words, tokenprint):
    if len(words) == 0:
        return None
    text = ' "' + ' '.join(words) + '", '
    text = text.replace('\\n ', '\\n')
    text = text.replace('\\t ', '\\t')
    return text

# Recursive walk through a nested tuple.
def walk(top, tokenprint=False, depth=0):
    out = []
    # Print the initial "(".
    if tokenprint:
        out.append('(')
    else:
        out.append('\n%s(' % (3*depth*' '))

    words = []
    counter = 0
    lastcount = len(top) - 1
    for obj in top:
        if type(obj) is TupleType:
            tokenprint2 = (len(obj) > 1 and obj[0] == TLMB and obj[-1] == TLME)
            # We are starting a new inner tuple so CombineWords.
            bigword = CombineWords(words, tokenprint2)
            words = []
            if bigword is not None:
                print 'L52', bigword
                out.append(bigword)
            out.extend(walk(obj, tokenprint2, depth+1))
        elif obj == '\\t':
            if keepNL:
                words.append('\\t')
        elif obj in ('nl1', 'nl2', 'nl3', 'nl4'):
            if keepNL:
                words.append('\\n')
        elif counter == 0:
            print 'L62', obj
            out.append('"%s",' % obj)
        elif tokenprint and counter == 1:
            print 'L65', obj
            out.append('"%s",' % obj)
        elif tokenprint and counter == lastcount or \
               obj in stop_tokens:
            bigword = CombineWords(words, tokenprint)
            words = []
            if bigword is not None:
                print 'L72', bigword
                out.append(bigword)
            print 'L74', obj
            out.append('"%s",' % obj)
        else:
            if obj in special_tokens and obj not in stop_tokens:
                words.append(special_dict[obj])
            else:
                words.append(obj)
        counter += 1

    bigword = CombineWords(words, tokenprint)
    if bigword is not None:
        print 'L85', bigword
        out.append(bigword)
    # Remove last comma in each tuple.
    if out[-1][-1] == ',':
        out[-1] = out[-1][:-1]
    if tokenprint:
        out.append('),')
    else:
        out.append('\n%s),' % (3*depth*' '))
    return out

keepNL = False

if len(sys.argv) != 2:
    print >> sys.stderr, 'program requires one argument'
    sys.exit(0)
root = sys.argv[1]

text = file(root + '.data', 'r').read().strip()
if text[0] == '(':
    text = text[:-1]
else:
    text = '(%s)' % text
text = text.replace('\t', '\\\\t')
data = eval(text)

out = walk(data)

print out
text = ''.join(out)[1:-1]
# A check.
try:
    eval(text)
except:
    print text
    raise

"""
# Prettier. Not machine readable.
text = text.replace(',(', ', (')
text = text.replace(',"', ', "')
text = text.replace('("TLMB827", ', '<<')
text = text.replace('("TLMB827",', '<<')
text = text.replace(', "TLME561")', '>>')
"""
file(root + '.clean', 'w').write(text)
