#! /usr/bin/env python

""" Returns C code with token information added.

Run "./insert_prints2.py" with "do_rules = NONE". Then "./moveit", make in
directory "grammars", and "./unparseit cvrest". Run this.

quotes              Leave "" around tokens and leave the commas between them.
origNL              Replace the strings '"nl1"', etc with "\n".
magic               Remove the magic tokens "TLMB827" and "TLME561" and
                    Use << ... >> for token prints.
bracket_ruleprints  Use [[ ... ]] for rule prints.
NL_for_ExternalDef  Start externalDef rules on new lines.
strip_externalList  Remove the (externalList ... ) that surrounds the
                    entire output
ats                 Remove the externalDef rule prints and separate
                    declarations by "@@@".
special_tokens      Use SEMI for ";", etc.
"""
root = 'cvrest'
"""
# Easy to read by Python eval.
quotes = True
origNL = True
magic = True
bracket_ruleprints = False
NL_for_ExternalDef = True
strip_externalList = False
ats = False
"""
"""
# Easy for human.
quotes = False
origNL = True
magic = False
bracket_ruleprints = True
NL_for_ExternalDef = True
strip_externalList = True
ats = True
"""
# Easy to read.
quotes = False
origNL = True
magic = False
bracket_ruleprints = False
NL_for_ExternalDef = True
strip_externalList = False
ats = False
special_tokens = True

text = file(root + '.data', 'r').read()
text = text.replace('\n', '')

# Every "," is followed by a space.
text = text.replace(', ', 'QQQ')
text = text.replace(',', ', ')
text = text.replace('QQQ', ', ')

# Remove quotes and commas.
if not quotes:
    text = text.replace('"', '')
    text = text.replace(',', '')

# Remove "nl1", etc.
if origNL:
    if quotes:
        text = text.replace('"nl1", ', '\n')
        text = text.replace('"nl2", ', '\n')
        text = text.replace('"nl3", ', '\n')
        text = text.replace('"nl4" ', '\n')
    else:
        text = text.replace('nl1 ', '\n')
        text = text.replace('nl2 ', '\n')
        text = text.replace('nl3 ', '\n')
        text = text.replace('nl4 ', '\n')

# Remove token print magic tokens.
if not magic:
    text = text.replace('(TLMB827 ', '<<')
    text = text.replace(' TLME561)', '>>')

# Each externalDef starts a new line.
"""
for i in range(5):
    text = text.replace('[[externalDef \n', '\n[[externalDef ')
text = text.replace(']] [[externalDef', ']]\n[[externalDef ')
"""
if NL_for_ExternalDef:
    if not quotes:
        for i in range(5):
            text = text.replace('(externalDef \n', '\n(externalDef ')
        text = text.replace(') (externalDef', ')\n(externalDef ')
    else:
        for i in range(5):
            text = text.replace('("externalDef", \n', '\n("externalDef",')
        text = text.replace(') ("externalDef"', ')\n("externalDef" ')

# Use [[ ...]] around rule prints.
if bracket_ruleprints:
    text = text.replace('(', '[[')
    text = text.replace(')', ']]')

if strip_externalList:
    if bracket_ruleprints:
        text = text.replace('[[externalList', '')
        text = text.strip()
        text = text[:-3]
    else:
        text = text.replace('(externalList', '')
        text = text.strip()
        text = text[:-2]

if ats:
    if bracket_ruleprints:
        text = text.replace('[[externalDef ', '@@@\n') + '\n@@@\n'
        text = text.replace(']] \n@@@', '\n@@@')
        text = text.replace(']]\n@@@', '\n@@@')
    else:
        text = text.replace('(externalDef ', '@@@\n') + '\n@@@\n'
        text = text.replace(') \n@@@', '\n@@@')
        text = text.replace(')\n@@@', '\n@@@')

text = text.replace('\t ', '\t')
text = text.replace('\n\t\n', '\n')

# Remove special tokens.
if not special_tokens:
    text = text.replace('SEMI', ';')
    text = text.replace('COMMA', ',')
    text = text.replace('LPARENS', '(')
    text = text.replace('RPARENS', ')')
text = text.replace(' \n', '\n')

file(root + '.pure', 'w').write(text)
