#! /usr/bin/env python

from __future__ import generators
import sys, treematch
from types import *

stop_tokens = ('SEMI', 'COMMA', 'LPARENS', 'RPARENS', '{', '}',
      'nl1', 'nl2', 'nl3', 'nl4', '\t')
remove_tokens = ('nl1', 'nl2', 'nl3', 'nl4', '\t')

def walktuple(top, depth=0):
    yield top, depth
    for obj in top:
        if type(obj) == TupleType:
            yield '(', depth
            for obj2, depth2 in walktuple(obj, depth+1):
                yield obj2, depth2
            yield ')', depth
        else:
            yield obj, depth

def tokenize(top):
    out = ['(']
    for obj2, depth in walktuple(top, 1):
        if type(obj2) != TupleType:
            out.append(obj2)
    out.append(')')
    return out

def CombineTokens(tokens):
    out = []
    words = []
    parens_last = False
    for token in tokens:
        if token in remove_tokens:
            continue
        if '(' in token or ')' in token or token in stop_tokens or parens_last:
            if len(words) > 0:
                phrase = ' '.join(words)
                out.append(phrase)
                words = []
            out.append(token)
        else:
            words.append(token)
        parens_last = ( '(' in token )
    if len(words) > 0:
        phrase = ' '.join(words)
        out.append(phrase)
    return out

def make_evalable(tokens):
    out = []
    for token in tokens:
        if token == '(':
            out.append('(')
        elif token == ')':
            out.append('), ')
        else:
            out.append('"%s", ' % token)
    text = ''.join(out)
    text = text[:-2]
    return text

def main(text):
    text = text[:-3]
    top = eval(text)
    pairs = []
    for obj in top[1:]:
        tokens = tokenize(obj)
        tokens = CombineTokens(tokens)
        evalable = make_evalable(tokens)
        source = treematch.gettokens(obj)
#        source = '"%s"' % treematch.gettokens_join(obj)
        pairtext = '(%s, %s)' % (evalable, source)
        pairs.append(pairtext)
    if len(pairs) == 1:
        text = '(' + pairs[0] + ',)'
    else:
        text = '(' + ',\n '.join(pairs) + ')'
    return text

if len(sys.argv) != 2:
    print >> sys.stderr, 'program requires one argument'
    sys.exit(0)
root = sys.argv[1]
text = file(root + '.data', 'r').read()
pairstext = main(text)
try:
    tup = eval(pairstext)
except:
    print >> sys.stderr, 'eval test failed'
    raise
text = treematch.prettyprint(tup)
file(root + '.clean', 'w').write(text)
