ctypedef int size_t
ctypedef unsigned char uchar

cdef extern from "stdlib.h":
    void* malloc(size_t size)
    void free(void* ptr)

cdef extern from "stdio.h":
    int printf(char* format, ...)

cdef extern from "string.h":
    int strcmp(char* s1, char* s2) 

cdef extern from "Python.h":
    void* PyMem_Malloc(size_t n)
    void* PyMem_Realloc(void *p, size_t n)
    void PyMem_Free(void *p)

    int PyCObject_Check(object p)
    object PyCObject_FromVoidPtr(void* cobj, void (*destr)(void *))
    object PyCObject_FromVoidPtrAndDesc(void* cobj, void* desc,
            void (*destr)(void *, void *))
    void* PyCObject_AsVoidPtr(object self)
    void* PyCObject_GetDesc(object self)

    int PyInt_Check(object p)
    int PyFloat_Check(object p)
    int PyString_Check(object p)
    object PyString_FromString(char *v)
    int PyInt_AsLong(object p)

# These 2 functions are NOT ENOUGH for pointers. "char*" has two standard
# meanings. "AAA*" may be a pointer to AAA or an array of AAA. Etc.
cdef wrap_ptr(void* p, char* ctypename):
    return PyCObject_FromVoidPtrAndDesc(p, ctypename, NULL)

cdef void* extract_ptr(object PY):
    if PY is None:
        return NULL
    return PyCObject_AsVoidPtr(PY)

def OpaquePointer_FromBasicType(object PY):
    cdef int i
    cdef int* ip
    if PyInt_Check(PY):
        # MEMORY LEAK.
        ip = <int*> PyMem_Malloc(sizeof(int))
        if ip == NULL:
            raise MemoryError
        ip[0] = PyInt_AsLong(PY)
        return wrap_ptr(ip, 'int*')
    else:
        raise TypeError, 'input must have type int, float or string'

def OpaquePointer_AsBasicType(PY):
    cdef void* p
    cdef char* ctypename
    cdef object s
    p = PyCObject_AsVoidPtr(PY)
    ctypename = <char*> PyCObject_GetDesc(PY)
    s = ctypename
    if s == 'char':
        return (<char *> p)[0]
    if s == 'int':
        return (<int*> p)[0]
    if s == 'float':
        return (<float *> p)[0]
    if s == 'double':
        return (<double *> p)[0]
    if s == 'string':
        return PyString_FromString(<char *> p)
    raise TypeError, 'CObject must wrap a pointer of a numerical type ' \
            'or a char*, type was %s' % ctypename

def CObject_ToPrettyString(object CO):
    cdef int i
    cdef void* p
    cdef char* s
    if not PyCObject_Check(CO):
        raise TypeError, 'not a CObject'
    p = PyCObject_AsVoidPtr(CO)
    s = <char*> PyCObject_GetDesc(CO)
    if p == NULL:
        val = "NULL"
    else:
        i = <int> p
        val = '0x%x' % i
    return '<CObject wrapping pointer to %s: %s>' % (s, val)
