typedef struct {} IplImage;
typedef struct {} CvSize;
typedef struct {} CvRect;
typedef struct {} CvMat;
typedef struct {} CvArr;
typedef struct {} CvMemStorage;
typedef struct {} CvSeq;
typedef struct {} IplConvKernel;
typedef struct {} CvScalar;
typedef struct {} CvPoint;
typedef struct {} CvPoint2D32f;
typedef struct {} CvMemStoragePos;
typedef struct {} CvSeqBlock;
typedef struct {} CvSeqWriter;
typedef struct {} CvSeqReader;
typedef struct {} CvSlice;
typedef struct {} CvSetElem;
typedef struct {} CvSet;
typedef struct {} CvGraph;
typedef struct {} CvGraphVtx;
typedef struct {} CvGraphEdge;


IplImage*  cvCreateImageHeader( CvSize size, int depth, int channels );

IplImage* cvInitImageHeader( IplImage* image, CvSize size, int depth,
                                       int channels, int origin,
                                       int align, int clear );

IplImage*  cvCreateImage( CvSize size, int depth, int channels );

void  cvReleaseImageHeader( IplImage** image );

void  cvReleaseImage( IplImage** image );

IplImage* cvCloneImage( const IplImage* image );

void  cvSetImageCOI( IplImage* image, int coi );

int  cvGetImageCOI( IplImage* image );

void  cvSetImageROI( IplImage* image, CvRect rect );

void  cvResetImageROI( IplImage* image );

CvRect cvGetImageROI( const IplImage* image );

CvMat*  cvCreateMatHeader( int rows, int cols, int type );

CvMat* cvInitMatHeader( CvMat* mat, int rows, int cols,
                                  int type, void* data,
                                  int step );

CvMat*  cvCreateMat( int rows, int cols, int type );

void  cvReleaseMatHeader( CvMat** mat );

void  cvReleaseMat( CvMat** mat );

CvMat* cvCloneMat( const CvMat* mat );

CvMat* cvGetSubArr( const CvArr* arr, CvMat* submat, CvRect rect );

CvMat* cvGetRow( const CvArr* arr, CvMat* submat, int row );

CvMat* cvGetCol( const CvArr* arr, CvMat* submat, int column );

CvMat* cvGetDiag( const CvArr* arr, CvMat* submat,
                            int diag);

/* ptr = &arr(idx) */
unsigned char* cvGetPtrAt( const CvArr* arr, int y, int x);

/* value = arr(idx) */
CvScalar cvGetAt( const CvArr* arr, int y, int x);

/* arr(idx) = value */
void cvSetAt( CvArr* arr, CvScalar value,
                        int y, int x);

/* Converts CvArr (IplImage or CvMat) to CvMat */
CvMat* cvGetMat( const CvArr* src, CvMat* header, int* coi);

/* Converts CvArr (IplImage or CvMat) to IplImage */
IplImage* cvGetImage( const CvArr* array, IplImage* img );

CvMat* cvReshape( const CvArr* array, CvMat* header,
                            int new_cn, int new_rows);

void  cvCreateData( CvArr* array );

void  cvReleaseData( CvArr* array );

void  cvSetData( CvArr* array, void* data, int step );

void cvGetRawData( const CvArr* array, unsigned char** data,
                             int* step, CvSize* roi_size );

CvSize cvGetSize( const CvArr* arr );

void  cvCopy( const CvArr* src, CvArr* dst,
                         const CvArr* mask );

void  cvSet( CvArr* arr, CvScalar scalar,
                        const CvArr* mask );

void cvLUT( const CvArr* srcarr, CvArr* dstarr,
             const CvArr* lutarr );

void  cvPyrUp( const CvArr* src, CvArr* dst,
                          int filter );

void  cvPyrDown( const CvArr* src, CvArr* dst,
                            int filter );

void cvPyrSegmentation( IplImage* src,
                               IplImage* dst,
                               CvMemStorage *storage,
                               CvSeq **comp,
                               int level, double threshold1,
                               double threshold2 );

void cvSobel( const CvArr* src, CvArr* dst,
                        int xorder, int yorder,
                        int apertureSize );

void cvLaplace( const CvArr* src, CvArr* dst,
                          int apertureSize );

typedef enum CvElementShape
{
    CV_SHAPE_RECT    = 0,
    CV_SHAPE_CROSS   = 1,
    CV_SHAPE_ELLIPSE = 2,
    CV_SHAPE_CUSTOM  = 100
}
CvElementShape;

IplConvKernel*  cvCreateStructuringElementEx( int  cols,    int rows,
                                                         int  anchorX, int anchorY,
                                                         CvElementShape shape,
                                                         int* values );

void  cvReleaseStructuringElement( IplConvKernel** element );

void  cvErode( const CvArr* src, CvArr* dst,
                          IplConvKernel* element,
                          int iterations);

void  cvDilate( const CvArr* src, CvArr* dst,
                           IplConvKernel* element,
                           int iterations );

typedef enum CvMorphOp
{
    CV_MOP_OPEN = 2,
    CV_MOP_CLOSE = 3,
    CV_MOP_GRADIENT = 4,
    CV_MOP_TOPHAT = 5,
    CV_MOP_BLACKHAT = 6
} CvMorphOp;

void  cvMorphologyEx( const CvArr* src, CvArr* dst,
                                 CvArr* temp, IplConvKernel* element,
                                 CvMorphOp operation, int iterations );

CvScalar  cvSum( const CvArr* array );

int  cvCountNonZero( const CvArr* array );

CvScalar  cvAvg( const CvArr* array, const CvArr* mask );

void  cvAvgSdv( const CvArr* array, CvScalar* mean, CvScalar* std_dev,
                           const CvArr* mask );

void  cvMinMaxLoc( const CvArr* array, double* min_val, double* max_val,
                              CvPoint* min_loc,
                              CvPoint* max_loc,
                              const CvArr* mask );

typedef struct CvMoments
{
    /* spatial moments */
    double  m00, m10, m01, m20, m11, m02, m30, m21, m12, m03;
    /* central moments */
    double  mu20, mu11, mu02, mu30, mu21, mu12, mu03;
    /* m00 != 0 ? 1/sqrt(m00) : 0 */
    double  inv_sqrt_m00;
} CvMoments;

void cvMoments( const CvArr* array, CvMoments* moments, int binary );

double  cvGetSpatialMoment( CvMoments* moments, int x_order, int y_order );
double  cvGetCentralMoment( CvMoments* moments, int x_order, int y_order );
double  cvGetNormalizedCentralMoment( CvMoments* moments,
                                              int x_order, int y_order );

typedef struct CvHuMoments
{
    double hu1, hu2, hu3, hu4, hu5, hu6, hu7; /* Hu invariants */
} CvHuMoments;

void cvGetHuMoments( CvMoments*  moments, CvHuMoments*  hu_moments );

double  cvNorm( const CvArr* imgA, const CvArr* imgB, int normType,
                           const CvArr* mask );

void  cvLine( CvArr* array, CvPoint pt1, CvPoint pt2,
                         double color, int thickness,
                         int connectivity );

void  cvLineAA( CvArr* array, CvPoint pt1, CvPoint pt2,
                           double color, int scale);

void  cvRectangle( CvArr* array, CvPoint pt1, CvPoint pt2,
                              double color, int thickness);

void  cvCircle( CvArr* array, CvPoint center, int radius,
                           double color, int thickness);

void  cvCircleAA( CvArr* array, CvPoint center, int radius,
                             double color, int scale );

void  cvEllipse( CvArr* array, CvPoint center, CvSize axes,
                            double angle, double startAngle, double endAngle,
                            double color, int thickness);

void  cvEllipseAA( CvArr* array, CvPoint center, CvSize axes,
                              double angle, double startAngle,
                              double endAngle, double color,
                              int scale );

void  cvFillConvexPoly( CvArr* array, CvPoint* pts, int npts, double color );

void  cvFillPoly( CvArr* array, CvPoint** pts,
                             int* npts, int contours, double color );

void  cvPolyLine( CvArr* array, CvPoint** pts, int* npts, int contours,
                             int closed, double color,
                             int thickness,
                             int connectivity );

void  cvPolyLineAA( CvArr* array, CvPoint** pts, int* npts, int contours,
                               int closed, double color, int scale );

typedef enum CvFontFace
{
    CV_FONT_VECTOR0 = 0
} CvFontFace;

typedef struct CvFont
{
    const int*  data; /* font data and metrics */
    CvSize      size; /* horizontal and vertical scale factors,
                         (8:8) fix-point numbers */
    int         italic_scale; /* slope coefficient: 0 - normal, >0 - italic */
    int         thickness; /* letters thickness */
    int         dx; /* horizontal interval between letters */
} CvFont;

void  cvInitFont( CvFont* font, CvFontFace font_face,
                             double hscale, double vscale,
                             double italic_scale,
                             int thickness );

void  cvPutText( CvArr* array, const char* text, CvPoint org,
                            CvFont* font, double color );

void  cvGetTextSize( const char* text_string, CvFont* font,
                                CvSize* text_size, int* ymin );

void  cvCvtColor( const CvArr* src, CvArr* dst, int colorCvtCode );

void  cvResize( const CvArr* src, CvArr* dst,
                           int method );

typedef struct CvLineIterator
{
    unsigned char* ptr;
    int  err;
    int  plus_delta;
    int  minus_delta;
    int  plus_step;
    int  minus_step;
} CvLineIterator;

int  cvInitLineIterator( const CvArr* array, CvPoint pt1, CvPoint pt2,
                                    CvLineIterator* lineIterator,
                                    int connectivity );

int  cvSampleLine( const CvArr* array, CvPoint pt1, CvPoint pt2, void* buffer,
                              int connectivity );

void  cvGetRectSubPix( const CvArr* src, CvArr* dst, CvPoint2D32f center );

void  cvGetQuadrangleSubPix( const CvArr* src, CvArr* dstarr,
                                        const CvArr* matrixarr,
                                        int fillOutliers,
                                        CvScalar fillvalue );

void  cvCvtPixToPlane( const void *src, void *dst0, void *dst1,
                                  void *dst2, void *dst3 );

void  cvCvtPlaneToPix( const void *src0, const void *src1,
                                  const void *src2, const void *src3,
                                  void *dst );

void  cvConvertScale( const CvArr *src, CvArr *dst,
                                 double scale,
                                 double shift );

void  cvConvertScaleAbs( const void *src, void *dst,
                                    double scale,
                                    double shift );

void  cvAbsDiff( const CvArr* srcA, const CvArr* srcB, CvArr* dst );

void  cvAbsDiffS( const CvArr* src, CvArr* dst, CvScalar value );

/* method for comparing two images */
typedef enum CvTemplMatchMethod
{
    CV_TM_SQDIFF        = 0,
    CV_TM_SQDIFF_NORMED = 1,
    CV_TM_CCORR         = 2,
    CV_TM_CCORR_NORMED  = 3,
    CV_TM_CCOEFF        = 4,
    CV_TM_CCOEFF_NORMED = 5
} CvTemplMatchMethod;

void  cvMatchTemplate( const CvArr* array, const CvArr* templ,
                                  CvArr* result, CvTemplMatchMethod method );

void  cvbFastArctan( const float* y, const float* x, float* angle, int len );

void  cvbCartToPolar( const float* y, const float* x,
                                float* magnitude, float* angle, int len );

void  cvbSqrt( const float* x, float* sqrt_x, int len );

int  cvCheckArr( const CvArr* arr, int flags,
                            double minVal, double maxVal);

void  cvbInvSqrt( const float* x, float* inv_sqrt_x, int len );

void  cvbReciprocal( const float* x, float* inv_x, int len );

void  cvbFastExp( const float* x, double* exp_x, int len );

void  cvbFastLog( const double* x, float* log_x, int len );

/* RNG state */
typedef struct CvRandState
{
    unsigned long    state;    /* RNG state (the current seed and carry)*/
    CvScalar  param[2]; /* parameters of RNG */
} CvRandState;

void  cvRandInit( CvRandState* state, double lower, double upper, int seed );

void  cvRandSetRange( CvRandState* state, double lower, double upper,
                                 int index );

void  cvbRand( CvRandState* state, float* dst, int len );

void  cvRand( CvRandState* state, CvArr* arr );

unsigned  cvRandNext( CvRandState* state );

void    cvUpdateMotionHistory( const CvArr* silhouette, CvArr* mhi,
                                          double timestamp, double mhiDuration );

void    cvCalcMotionGradient( const CvArr* mhi, CvArr* mask, CvArr* orientation,
                                         double maxTDelta, double minTDelta,
                                         int aperture_size );

double  cvCalcGlobalOrientation( const CvArr* orientation, const CvArr* mask,
                                            const CvArr* mhi, double curr_mhi_timestamp,
                                            double mhi_duration );

CvSeq*  cvSegmentMotion( CvArr* mhi, CvArr* seg_mask,
                                    CvMemStorage* storage,
                                    double timestamp, double seg_thresh );

void  cvAcc( const CvArr* image, CvArr* sum,
                        const CvArr* mask );

void  cvSquareAcc( const CvArr* image, CvArr* sqSum,
                              const CvArr* mask );

void  cvMultiplyAcc( const CvArr* imgA, const CvArr* imgB, CvArr* acc,
                                const CvArr* mask );

void  cvRunningAvg( const CvArr* imgY, CvArr* imgU, double alpha,
                               const CvArr* mask );

CvMemStorage*  cvCreateMemStorage( int block_size);

CvMemStorage*  cvCreateChildMemStorage( CvMemStorage* parent );

void  cvReleaseMemStorage( CvMemStorage** storage );

void  cvClearMemStorage( CvMemStorage* storage );

void  cvSaveMemStoragePos( CvMemStorage* storage, CvMemStoragePos* pos );

void  cvRestoreMemStoragePos( CvMemStorage* storage, CvMemStoragePos* pos );

CvSeq*  cvCreateSeq( int seq_flags, int header_size,
                             int elem_size, CvMemStorage* storage );

void  cvSetSeqBlockSize( CvSeq* seq, int delta_elements );

char*  cvSeqPush( CvSeq* seq, void* element);

char*  cvSeqPushFront( CvSeq* seq, void* element);

void  cvSeqPop( CvSeq* seq, void* element );

void  cvSeqPopFront( CvSeq* seq, void* element );

void  cvSeqPushMulti( CvSeq* seq, void* elements, int count );

void  cvSeqPopMulti( CvSeq* seq, void* elements, int count );

char*  cvSeqInsert( CvSeq* seq, int before_index,
                               void* element);

void  cvSeqRemove( CvSeq* seq, int index );

void  cvClearSeq( CvSeq* seq );

char*  cvGetSeqElem( CvSeq* seq, int index, CvSeqBlock** block );

int  cvSeqElemIdx( CvSeq* seq, void* element, CvSeqBlock** block );

void  cvStartAppendToSeq( CvSeq* seq, CvSeqWriter* writer );

void  cvStartWriteSeq( int seq_flags, int header_size,
                               int elem_size, CvMemStorage* storage,
                               CvSeqWriter* writer );

CvSeq*  cvEndWriteSeq( CvSeqWriter* writer );

void   cvFlushSeqWriter( CvSeqWriter* writer );

void cvStartReadSeq( CvSeq* seq, CvSeqReader* reader, int reverse );

int    cvGetSeqReaderPos( CvSeqReader* reader );

void   cvSetSeqReaderPos( CvSeqReader* reader, int index,
                                  int is_relative);

void*  cvCvtSeqToArray( CvSeq* seq, CvArr* array,
                                   CvSlice slice );

void  cvMakeSeqHeaderForArray( int seq_type, int header_size,
                                          int elem_size, CvArr* array, int total,
                                          CvSeq* seq, CvSeqBlock* block );

void  cvChangeSeqBlock( CvSeqReader* reader, int direction );
void  cvCreateSeqBlock( CvSeqWriter* writer );

CvSet*   cvCreateSet( int set_flags, int header_size,
                              int elem_size, CvMemStorage* storage );

int   cvSetAdd( CvSet* set_struct, CvSetElem* element,
                           CvSetElem** inserted_element );

void   cvSetRemove( CvSet* set_struct, int index );

CvSetElem*  cvGetSetElem( CvSet* set_struct, int index );

void   cvClearSet( CvSet* set_struct );

CvGraph*   cvCreateGraph( int graph_flags, int header_size,
                                  int vtx_size, int edge_size,
                                  CvMemStorage* storage );

int   cvGraphAddVtx( CvGraph* graph, CvGraphVtx* vertex,
                                CvGraphVtx** inserted_vertex );

void   cvGraphRemoveVtx( CvGraph* graph, int index );
void   cvGraphRemoveVtxByPtr( CvGraph* graph, CvGraphVtx* vtx );

int  cvGraphAddEdge( CvGraph* graph,
                                int start_idx, int end_idx,
                                CvGraphEdge* edge,
                                CvGraphEdge** inserted_edge );

int  cvGraphAddEdgeByPtr( CvGraph* graph,
                               CvGraphVtx* start_vtx, CvGraphVtx* end_vtx,
                               CvGraphEdge* edge,
                               CvGraphEdge** inserted_edge );

void   cvGraphRemoveEdge( CvGraph* graph, int start_idx, int end_idx );
void   cvGraphRemoveEdgeByPtr( CvGraph* graph, CvGraphVtx* start_vtx,
                                          CvGraphVtx* end_vtx );

CvGraphEdge*  cvFindGraphEdge( CvGraph* graph, int start_idx, int end_idx );
CvGraphEdge*  cvFindGraphEdgeByPtr( CvGraph* graph, CvGraphVtx* start_vtx,
                                               CvGraphVtx* end_vtx );

void  cvClearGraph( CvGraph* graph );

int  cvGraphVtxDegree( CvGraph* graph, int vtx_idx );
int  cvGraphVtxDegreeByPtr( CvGraph* graph, CvGraphVtx* vtx );

